//file avrmt128tst.c
//test olimex avrmt128 board (uses rxd1 and txd1 as main rs232)
//cpu: atmel atmega128 16mhz 115200

//Jan 16 08 Bob G recompile with 7.15

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include <macros.h> //stackcheck
#include "mt128lcd.h"
#include <iom128v.h>

#define INTR_OFF() asm("CLI")
#define INTR_ON()  asm("SEI")

//------vars in bss-------
unsigned char jnk;
unsigned char dispon;
unsigned char teston;
unsigned char os1sec;
unsigned char passcnt;
unsigned char deltatocs;
unsigned char secs,mins,hrs;
unsigned char fd; //0=usart 0, 1=usart 1
unsigned int addat10[8];  
unsigned char tocs,tocsl; //timer0 output compare 1ms 
unsigned int tics,ticsl; //prescaled timer tics
unsigned int t;
char osaccum,deltat,hr,min,sec;

//------rom---------
const unsigned char banner[]={"avrmt128tst  Jan 16 08    q for menu\n"};

//-----------------------
void port_init(void){
  PORTA = 0x00;
  DDRA  = 0x40; //pa6 out relay
 
  PORTB = 0xF0; //spi
  DDRB  = 0xF7; //last 3 outs
 
  PORTC = 0xFF; //lcd
  DDRC  = 0xFF; //all outs
 
  PORTD = 0xFB; //
  DDRD  = 0x4c; //pd6,3,2 out
 
  PORTE = 0xFD; //serial port 
  DDRE  = 0x02;
 
  PORTF = 0x00; //a/d (pullups off)
  DDRF  = 0x00; //all ins
 
  PORTG = 0x1F;
  DDRG  = 0x00; //all ins
}

//---------------------
//SPI initialisation
// clock rate: 115200hz
void spi_init(void){
  SPCR = 0x50; //setup SPI (slowest clock rate! 0x50 fastest)
  SPSR = 0x00; //setup SPI
}

//--------------------
//UART0 initialisation
// desired baud rate:
// actual: baud rate:
// char size: 8 bit
// parity: Disabled
void uart0_init(void){
  UCSR0B = 0x00; //disable while setting baud rate
  UCSR0A = 0x00;
  UCSR0C = 0x06;
  UBRR0L = 0x07; //set baud rate lo (0x19=38400 @16) 9=115200@18.432 7=115@16
  UBRR0H = 0x00; //set baud rate hi
  UCSR0B = 0x18; //rx int on   0x18=off
}

//--------------------
//UART1 initialisation
// desired baud rate: 
// actual: baud rate:
// char size: 8 bit
// parity: Disabled
void uart1_init(void){
  UCSR1B = 0x00; //disable while setting baud rate
  UCSR1A = 0x00;
  UCSR1C = 0x06;
  UBRR1L = 0x07; //set baud rate lo (19=38400@16) 9=115200@18 
  UBRR1H = 0x00; //set baud rate hi
  UCSR1B = 0x18; //rx int on   0x18=off
}

//--------------------
//ADC initialisation
// Conversion time: 112uS
void adc_init(void){
  ADCSRA = 0x00; //disable adc
  ADMUX  = 0x00; //select adc input 0
  ACSR   = 0x80;
  ADCSRA = 0x87;
}

//-------------------------------
//TIMER0 initialize - prescale:1024
// WGM: Normal
// desired value: 1Hz
// actual value:  1.000Hz (0.0%)
void timer0_init(void){
  TCCR0 = 0x00; //stop
  ASSR  = 0x08; //set async mode
  TCNT0 = 0xE0; //set count
  OCR0  = 0x20;
  TCCR0 = 0x07; //start timer
}

//---------------------------
void init_devices(void){
//call this routine to initialise all peripherals

  CLI(); //disable all interrupts
  XDIV  = 0x00; //xtal divider
  XMCRA = 0x00; //external memory
  port_init();
  spi_init();
  uart0_init();
  uart1_init();
  adc_init();
  timer0_init();
 
  MCUCR = 0x00; //no ext mem
  EICRA = 0x00; //extended ext ints
  EICRB = 0x00; //extended ext ints
  EIMSK = 0x00;
  TIMSK = 0x02; //timer interrupt sources  timer0 oc
  ETIMSK = 0x00; //extended timer interrupt sources
  SEI(); //re-enable interrupts
 //all peripherals are now initialised
}

//file mt128lcd.c
#define RSLO() PORTC &= ~0x01
#define RSHI() PORTC |=  0x01

#define RWLO() PORTC &= ~0x02
#define RWHI() PORTC |=  0x02

#define ENLO() PORTC &= ~0x04
#define ENHI() PORTC |=  0x04

//------------------------
void lcddel10us(int n){
//delay 10 usec
int j;

  while(n--) {
    j=28;
    while(j--){}; //2800=1ms, 280=100usec, 28=10usec
  }
}

//--------------------
void putnib(char c){
//lo bytes of c to hi bits of port

  PORTC = (PORTC & 0x0f)|(c << 4);
  ENHI();
  asm("nop"); //8x62=500ns
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  ENLO();
}

//--------------------
void lcdputbyte(unsigned char c){
//either data or cmd byte, previously set

  RWLO(); //write
  putnib(c >> 4);    //hi nib
  putnib(c & 0x0f);  //lo nib
  lcddel10us(5);
}

//--------------------
void lcdcmdbyte(unsigned char c){
//cmd byte

  RSLO(); //command
  lcdputbyte(c);
}

//--------------------
void lcddatabyte(unsigned char c){
//data byte

  RSHI();  //data
  lcdputbyte(c);
}

//----------------
void lcdcls(void){
//clear lcd screen    1.64ms

  lcdcmdbyte(0x01);
  lcddel10us(165);
}

//--------------------------------------
void lcdgotoxy(unsigned char column, unsigned char line){
//position cursor  top line chars 0-39, 2nd line 40-79
unsigned char addr;

  addr = (line * 64) + column;
  lcdcmdbyte(addr | 0x80);
}

//----------------
void lcdcursoron(void){
//cursor on

  lcdcmdbyte(0x0e);
}

//----------------
void lcdcursoroff(void){
//cursor off

  lcdcmdbyte(0x0c);
}

//------------------
void lcdputc(char c){
//put c

  lcddatabyte(c);
}

//------------------
void lcdcputs(const char *ptr){
//put rom string at ptr

  while(*ptr != 0x00){
    lcdputc(*ptr++);
  }  
}

//------------------
void lcdputs(char *ptr){
//put ram string at ptr

  while(*ptr != 0x00){
    lcdputc(*ptr++);
  }  
}

//------------------
void lcdinit(void){
//init lcd 4 bit mode

  lcddel10us(1500); //15000us
  lcdcmdbyte(0x38); //8 bit 2 lines
  lcddel10us(410);  //4100us
  lcdcmdbyte(0x28); //4 bit 2 lines
  lcddel10us(10);   //100us
  lcdcmdbyte(0x28); //4 bit 2 lines
  lcdcmdbyte(0x0c); //display on    40usec
  lcdcmdbyte(0x06); //cursor on, increment  40usec
  lcdcmdbyte(0x80); //cg addr 0     40usec
  lcdcmdbyte(0x40); //data addr 0   40usec
}

//--------------------
void initvars(void){
//init some vars

  MCUCSR=0x80;
  MCUCSR=0x80; //jtag disable
  fd=1;        //use serial port 1 as primary serial port
}

//------------------
void relayon(void){
//relay   pa6

  PORTA |= 0x40;
}

//------------------
void relayoff(void){
//relay off

  PORTA &= ~0x40;
}

//------RS232-------------------
unsigned char kbhit0(void){
//return nonzero if char waiting   polled version
unsigned char b;

  b=0;
  if(UCSR0A & (1<<RXC0)) b=1;
  return b;
}

//-------------------------
unsigned char kbhit1(void){
//return nonzero if char waiting   polled version
unsigned char b;

  b=0;
  if(UCSR1A & (1<<RXC0)) b=1;
  return b;
}

//-------------------------
unsigned char kbhit(void){
//return nonzero if char waiting   polled version
unsigned char b;

  if(fd)
    b=kbhit1();
  else	
    b=kbhit0();
  return b;
}

#if 0
#define NUMCHARS 64
unsigned char rxbuf[NUMCHARS];   //rs232 interrupt buffer
unsigned char rxindx,rxondx;  
 
//-------------------------
unsigned char kbhit(void){
//return nonzero if char waiting  intr version

  return(rxindx != rxondx);
}

//-----------------------
int getchar(void){
//intr version... overrides version in library
char c;

//  while(!kbhit()){}; //wait for char  subroutine call version
  while(rxindx==rxondx){}; //inline version... no subroutine call
  c=rxbuf[rxondx++]; //get char from rxbuf
  if(rxondx >= NUMCHARS) rxondx=0; 
  return c;  
}

//-------------------------
#pragma interrupt_handler uart0_rx_isr:iv_USART0_RX
void uart0_rx_isr(void){
//uart has received a character in UDR
//32 inst at 14 inst/usec= 2usec, 2.5% of the cpu tine
char c;

  c=UDR1;            //get char from usart
  rxbuf[rxindx++]=c; //put char in rxbuf
  if(rxindx >= NUMCHARS) rxindx=0;   
}
#endif

//-----------------------
int getchar0(void){
//polled version... overrides version in library
char c;

  while(!kbhit()){}; //wait for char  subroutine call version
  c=UDR0;            //get char from usart
  return c;  
}

//-----------------------
int getchar1(void){
//polled version... overrides version in library
char c;

  while(!kbhit()){}; //wait for char  subroutine call version
  c=UDR1;            //get char from usart
  return c;  
}

//-----------------------
int getchar(void){
//polled version... overrides version in library
char c;

  while(!kbhit()){}; //wait for char  subroutine call version
  if(fd)
    c=UDR1;            //get char from usart
  else	
    c=UDR0;            //get char from usart
  return c;  
}

//-----------------------
int putc0(char c){
//put char to usart1... dont add lf after cr

  while((UCSR0A & 0x20)==0){};	//0x20 is UDRE0. ==0 means data register full
  UDR0=c; //send char
  return c;
}

//-----------------------
int putc1(char c){
//put char to usart1... dont add lf after cr

  while((UCSR1A & 0x20)==0){};	//0x20 is UDRE0. ==0 means data register full
  UDR1=c; //send char
  return c;
}

//-----------------------
int putc(char c){
//put char to usart1... dont add lf after cr

  if(fd){
    while((UCSR1A & 0x20)==0){};	//0x20 is UDRE0. ==0 means data register full
    UDR1=c; //send char
  }else{
    while((UCSR0A & 0x20)==0){};	//0x20 is UDRE0. ==0 means data register full
    UDR0=c; //send char
  }	
  return c;
}

//----------------------  
int putchar(char c)	{
//adds lf after cr

  if(c=='\n'){
    putc('\r');
  }  
  putc(c);
  return c;
}

//---------------
void crlf(void){
//send cr then lf

  putchar('\n');
}
  
//----------------------
void space(void){
//send space

  putchar(' ');
}

#if 0
//-------------------------
unsigned char kbhit1(void){
//return nonzero if char waiting in uart1 cue intr version
unsigned char b;

  b=rx1indx != rx1ondx;
  return b;
}

//-----------------------
int getchar1(void){
//get char from uart1 cue...intr version...
char c;

  while(!kbhit1()){}; //wait for char
  c=rx1buf[rx1ondx++]; //get char from rxbuf
  if(rx1ondx >= NUMCHARS) rx1ondx=0;
  return c;  
}

//----------------------------------------
#pragma interrupt_handler uart1_rx_isr:iv_USART1_RX
void uart1_rx_isr(void){
//uart1 has received a character in UDR1 RS422
char c;

  c=UDR1;            //get char from usart
  rx1buf[rx1indx++]=c; //put char in rxbuf
  if(rx1indx >= NUMCHARS) rx1indx=0;  
}

//-----------------------
int putc1(unsigned char c){
//put char to usart1... dont add lf after cr

  while((UCSR1A & 0x20)==0){};	//0x20 is UDRE0. ==0 means data register full
  UDR1=c; //send char
  return c;
}
#endif

#define ESC 0x1b
//----------------------  
void gotoxy(int x, int y){
//ansi cursor positioning sequence E[y;xH

  putc(ESC);
  putc('[');
  cprintf("%d",y);
  putc(';');
  cprintf("%d",x);
  putc('H');
}

//----------------------
void clrscr(void){
//clear ansi terminal screen

  putc(ESC);
  putc('[');
  putc('2');
  putc('J');
}

//------------------
void initscreen(void){
//clear screen and print banner

  clrscr();
  cprintf(banner);
}

#if 0 //use with small printf checked
//----------------------
void putnibhex(unsigned char c){
//output bin nib c in hex

  c+=0x30;           //cvt to ascii
  if(c > 0x39) c+=7; //handle a-f
  putc(c);
}

//-----------------------
void putbytehex(unsigned char c){
//output c as 2 hex chars
unsigned char nib;

  nib=(c & 0xf0) >>4;
  putnibhex(nib);
  nib=c & 0x0f;
  putnibhex(nib);
}

//-----------------------
void putwordhex(unsigned int w){
//output w as 4 hex chars

  putbytehex(w >> 8);    //hi byte
  putbytehex(w && 0xff); //lo byte
}
#endif

//------debug subs-----------
void dump256ram(unsigned char *n){
//dump 16 rows of 16 bytes
unsigned char r,c;
unsigned char *p, *pa, ch;

  p=n;
  cprintf("\n      ");
  for(c=0; c<16; c++){
    cprintf("%02x ",c);      //header
    if(c==7) cprintf(" ");
  }  
  cprintf("\n\n");
  for(r=0; r<16; r++){
    cprintf("%04x  ",p);     //print addr  at beg of line
    pa=p;                   //remember p for ascii
    for(c=0; c<16; c++){
      cprintf("%02x ",*p++); //print hex
      if(c==7) cprintf(" ");
    }
    for(c=0; c<16; c++){
      ch=*pa++;
      if((ch > 0x20) && (ch !=0x0a) && (ch != 0x8a)) //if printing char
        cprintf("%c",ch);   //print ascii
      else  
        cprintf("."); 
      if(c==7) cprintf(" ");
    }
    cprintf("\n");
  }
}

//----------------------------
void dump256rom(const unsigned char *n){
//dump 16 rows of 16 bytes
unsigned char r,c;
const unsigned char *p, *pa;
unsigned char ch;

  p=n;
  cprintf("\n      ");
  for(c=0; c<16; c++){
    cprintf("%02x ",c);      //header
    if(c==7) cprintf(" ");
  }  
  cprintf("\n\n");
  for(r=0; r<16; r++){
    cprintf("%04x  ",p);     //print addr  at beg of line
    pa=p;                   //remember p for ascii
    for(c=0; c<16; c++){
      cprintf("%02x ",*p++); //print hex
      if(c==7) cprintf(" ");
    }
    for(c=0; c<16; c++){
      ch=*pa++;
      if((ch > 0x20) && (ch !=0x0a) && (ch != 0x8a)) //if printing char
        cprintf("%c",ch);   //print ascii
      else  
        cprintf("."); 
      if(c==7) cprintf(" ");
    }
    cprintf("\n");
  }
}

//-------------------------
unsigned char getche(void){
//get and echo a char from rs232
char c;

  c=getchar();
  putchar(c);
  return(c);
}

//-------------------------
unsigned char gethex(void){
//return a hex char from rs232
unsigned char b,c;

  b=0xff; //error return value
  c=toupper(getche());
  if(isxdigit(c)) b=c-0x30; //if c ok, cvt ascii digit to binary
  if((c>='A') && (c<='F')) b-=7;     //if c hexcvt ascii A to binary 10
  return(b);
}

//---------------------------
unsigned char getbyte(void){
//get 2 nibbles, return a binary byte from rs232
unsigned char n1,n2;

  n1=gethex();
  n2=gethex();
  return((n1 << 4) + n2);
}

//----------------------------
unsigned int getaddr(void){
//return addr from rs232
unsigned int th,tl;

  th=getbyte();
  tl=getbyte();
  return((th << 8) + tl);
}

#if 0 //getdec number
//---------------------
unsigned char getdecdig(void){
//return decimal digit 0..9 from rs232
//return ff (-1) of non hex, like carriage return
unsigned char b,c;

  b=0xff; //error return value
  c=toupper(getche());
  if(isdigit(c)) b=c-0x30; //if c ok, cvt ascii digit to binary
  return(b);
}

//-----------------------
unsigned char getnum(void){
//return number 0..255 from rs232  -1 to exit
char dig,n;

  dig=getdecdig();
  if(dig == -1) return dig; 
  n=dig;
//  if(n>2) n=2;
  dig=getdecdig();
  if(dig == -1) return dig; 
  n=n*10+dig;
//  if(n>25) n=25;
  dig=getdecdig();
  if(dig == -1) return dig; 
  n=n*10+dig;
  if(n>255) n=255;
  return(n);
}
#endif

//------------------
void examineram(void){
//ask for mem range in ram and dump
unsigned char *from;
unsigned char c;

  cprintf("from:");
  from=(unsigned char *)getaddr();
  while(c != 'q'){
    dump256ram(from);
    cprintf("np or q...");
    c=getchar();
    if(c=='n') from+=0x100;
    if(c=='p') from-=0x100;
  }  
}

//------------------
void examinerom(void){
//ask for mem range in rom and dump
const unsigned char *from;
unsigned char c;

  cprintf("from:");
  from=(const unsigned char *)getaddr();
  while(c != 'q'){
    dump256rom(from);
    cprintf("np or q...");
    c=getchar();
    if(c=='n') from+=0x100;
    if(c=='p') from-=0x100;
  }  
}

//------------------
void deposit(void){
//ask for addr and data
unsigned char *at;
unsigned char c;
unsigned char nh,nl;

  cprintf(" at:");
  at=(unsigned char *)getaddr();
  while(1){
    cprintf(" %02x ",*at);
    nh=gethex();
    if(nh==0xff) return;
    nl=gethex();
    if(nl==0xff) return;
    c=((nh << 4) | nl);
    *at++=c;
  }
}

//------------------
void fill(void){
//ask for mem range and fill char and fill
unsigned char *from, *to, with;

  cprintf("  from:");
  from=(unsigned char *)getaddr();
  cprintf(" to:");
  to=(unsigned char *)getaddr();
  cprintf(" with:");
  with=getbyte();
  memset(from,with,to-from);
}

//-----------------------------
void (* fn)(void); //fn prototype

void dojsr(void){
//ask for addr, jsr to it
const unsigned char *to;

  cprintf(" to:");
  to=(const unsigned char *)getaddr();
  fn=(void *)to;
  (*fn)();  //call function fn and returns
}

//------------------
void debugmenu(void){
//monitor type cmds

  cprintf("Debug cmds:\n");
  cprintf(" e examine ram\n");
  cprintf(" c examine rom\n");
  cprintf(" d deposit\n");
  cprintf(" f fill\n");
  cprintf(" j jsr\n");
  cprintf(" q quit\n");
}

//------------------
void debugloop(void){
//examine and deposit mem regs
char c;

  debugmenu();
  while(c != 'q'){
    crlf();
    cprintf("cmd:");
    c=getchar();
    crlf();
    switch(c){
      case 'e': examineram(); break;
      case 'c': examinerom(); break;
      case 'd': deposit(); break;
      case 'f': fill();    break;
      case 'j': dojsr();   break;
      default: debugmenu();
    }
  }
}
//----end of debug subs----------

#if 0
//-------------------------------
char readspi(void){
//return char from spi
char cnt,stat;

  cnt=0;
  do{
    stat=SPSR;
    cnt++;
    if(cnt==255){
      cprintf("spi rcv timeout ");
      break;
    }
  }while((stat & 0x80)==0);  //keep checking until done bit
  return(SPDR); //clears done flag
}

//--------------------------
void spiout8(char c){
//send c out spi
char cnt,stat;

  SPDR=c; //send data
  cnt=0;
  do{
    stat=SPSR;
	cnt++;    
    if(cnt==255){
      cprintf("spi xmt timeout! ");
      break;
    }
  }while((stat & 0x80)==0);
}
#endif

//----------------
void _StackOverflowed(char n){
//called from stackcheck

  cprintf("overflow! %d",n);
  while(1){}; //hang up
}

//-------------------
void delnms(int n){
//delay n ms
int x;

  while(n--){
//    x=2600;       //empirically determined fudge factor for 16 MHz
      x=2800;       //empirically determined fudge factor for 18 MHz
    while(x--);
  }
}

//-------------------
void del100us(int n){
//delay n times 100us
int x;

  while(n--){
    x=260; 
    while(x--);
  }
}

//------------------
void beep(void){
//pe45= xx10xxxx or xx01xxxx  10ms at 1khz
char n;
  
  n=10;
  while(n--){
    PORTE=(PINE &= ~0x30) | 0x10;
    del100us(5);
    PORTE=(PINE &= ~0x30) | 0x20;
    del100us(5);
  }
}

//---------------------
int readadchan(char n){
//read ch n of internal 10 bit a/d

  ADMUX=n;       //select channel n
  ADCSRA |=0x40; //init conversion
  while((ADCSRA & 0x40) !=0){}; //wait for conv complete
  return ADC;
}

//--------------------
void readnextchan(void){
//read next 10 bit chan
static char chan;

  chan++;
  if(chan >= 8) chan=0;
  addat10[chan]=readadchan(chan);
}

//--------------------
void readadloop(void){
//read internal 10 bit ad 0-0x3ff
char c,n;

  cprintf("10 bit a/d\n");
  cprintf("0   1   2   3   4   5   6   7 \n");
  c=0;
  while(c!='q'){
    if(kbhit()){
      c=getchar();
    }  
	  for(n=0; n < 8; n++){
	    addat10[n]=readadchan(n);
      cprintf("%03x ",addat10[n]);
	  }
	  cprintf("\r");  
  }
}

//-----------------
void getstr(char *p){
//get a string and echo
char c;  

  do{
    c=getche();
    *p++=c;
  }while(c !=0x0d);
  p--;     //roll p back
  *p=0x00; //add null over cr
}

char pina;
char bt,br,bl,bb,bc;
char anybut;
//--------------------------
char kphit(void){
//button on keypad hit

  pina=PINA & 0x1f;
  return (pina != 0x1f);
}

//----------------------
char getkp(void){
//get button from keypad
char cp;

  while(!kphit()){};
	cp=pina & 0x1f;
  return cp;
}

//-----------------------------
void butloop(void){
//read buttons
char c,cp;

  cprintf("button loop  q to quit\n");
  while(c != 'q'){
    if(kbhit()){    //if char waiting from pc
	    c=getchar();
	  }
    cp=getkp();
	  bt=!(cp & 0x01);
	  br=!(cp & 0x02);
	  bl=!(cp & 0x04);
	  bb=!(cp & 0x08);
	  bc=!(cp & 0x10);
    cprintf("%02x %02x %02x %02x %02x \r",bt,br,bl,bb,bc);
    delnms(100);
  }
}

//-----------------
//1 2 3 4 5
//*   * *   //0x16
//    * *   //0x06
//*   * *
//    * *
//*   * *
//    * *
//*   * *
//    * *

const char vuchar[8][8]={
//0    1    2    3    4    5    6    7  <-top of char on left
0x10,0x00,0x10,0x00,0x10,0x00,0x10,0x00, //0
0x10,0x00,0x10,0x00,0x10,0x00,0x10,0x06, //1
0x10,0x00,0x10,0x00,0x10,0x00,0x16,0x06, //2
0x10,0x00,0x10,0x00,0x10,0x06,0x16,0x06, //3
0x10,0x00,0x10,0x00,0x16,0x06,0x16,0x06, //4
0x10,0x00,0x10,0x06,0x16,0x06,0x16,0x06, //5
0x10,0x00,0x16,0x06,0x16,0x06,0x16,0x06, //7
0x10,0x06,0x16,0x06,0x16,0x06,0x16,0x06};//8

//-----------------
void initcgram(void){
//send 8 vu ladder chars to character generator ram 
char c,n;

  for(c=0; c<8; c++){   //8 chars
    for(n=0; n<8; n++){ //8 bytes/char
      lcdcmdbyte(0x40 + c*8 + n); //cg addr
      lcddatabyte(vuchar[c][n]);
      lcddel10us(15);
    }  
  }
  cprintf("\n");
}

//------------------
void vuloop(void){
//type to lcd
char c,n;

  cprintf("vu loop     q to quit\n");
  initcgram();
  n=0;
  while(c != 'q'){
    c=getche();
    n++;
    if(n==8) n=0;
    lcdgotoxy(1,1);
    lcdputc(n);
    cprintf("%02x ",n);
  }
}

char fader[16];
char level[16];
char curch;
//----------------
void initfaders(void){
//init faders
char n;

  for(n=0; n<16; n++){
    fader[n]=rand() % 8;
  }
}

//----------------
void drawfaders(void){
//draw faders
char n;

  for(n=0; n<16; n++){
    lcdgotoxy(n+1,2);  //chan 1-16
    lcdputc(fader[n]); //0-7
  }
}

//----------------
void initvus(void){
//init vus
char n;

  for(n=0; n<16; n++){
    level[n]=rand() % 8; //0-7
  }
}

//----------------
void drawvus(void){
//draw vus
char n;

  for(n=0; n<16; n++){
    lcdgotoxy(n+1,1);  //chan 1-16
    lcdputc(level[n]); //level 0-7
  }  
}

//------------------
void faders(void){
//16 faders on bottom line 16 vus on top line
char c,cp,cpl,cpos;

  cprintf("faders     q to quit\n");
  initfaders();
  drawfaders();
  initvus();
  drawvus();
  while(c != 'q'){
    if(kbhit()){
      c=getchar();
    }
    cp=0;
    if(kphit()){
      cp=getkp();
      cpos=cp != cpl;
      cpl=cp;
      bt=br=bl=bb=bc=0; //5 buttons
      if(cpos){
       	bt=!(cp & 0x01);
	      br=!(cp & 0x02);
	      bl=!(cp & 0x04);
	      bb=!(cp & 0x08);
	      bc=!(cp & 0x10);
        cprintf("    %02x\r",cp);
      }  
      if(br){
        curch++;
        if(curch > 15) curch=15;
        lcdgotoxy(curch+1,2);
        cprintf("%2d\r",curch);
      }  
      if(bl){
        curch--;
        if(curch < 0) curch=0;
        lcdgotoxy(curch+1,2);
        cprintf("%2d\r",curch);
      }  
      if(bt){
        fader[curch]++;
        if(fader[curch] > 15) fader[curch]=15;
        lcdcursoron();
        drawfaders();
        cprintf("%2d\r",fader[curch]);
      }  
      if(bb){
        fader[curch]--;
        if(fader[curch] < 0) fader[curch]=0;
        lcdcursoron();
        drawfaders();
        cprintf("%2d\r",fader[curch]);
      }  
    }//kphit
    lcdcursoroff();
    initvus();
    drawvus();
    delnms(200);  
  }
}

//------------------
void term(void){
//type to lcd 1st line is 1-63   2nd line is 64-127
char c,col;

  cprintf("term mode q to quit\n");
  lcdcls();
  col=0;
  while(c != 'q'){
    c=getche();
    lcdputc(c);
    col++;
    if(col==16){
      col=0;
      lcdgotoxy(1,2);
    }
  }
}

char teststring[16];
//-----------------
void walk(void){
//walk chars thru lcd
char c,b,n;

  cstrcpy(teststring,"1234567890");  
  while(c !='q'){
    if(kbhit()){
	    c=getchar();
	  }
    lcdputs(teststring);
	  for(n=1; n<10; n++){
	    b=teststring[n];
	    teststring[n-1]=b;
	  }
    printf(teststring);
	  cprintf("\n");	
	  delnms(300);
  }
}

//------------------
void lcdmenu(void){
//lcd menu

  cprintf("lcd cmds\n");
  cprintf(" c clear\n");
  cprintf(" w walk\n");
  cprintf(" t term\n");
  cprintf(" v vuloop\n");
  cprintf(" f faders\n");
}

//------------------
void lcdloop(void){
//examine and deposit mem regs
char c;

  cprintf("lcd loop    q=quit\n");
  while(c != 'q'){
    lcdmenu();
    cprintf("cmd:");
    c=getchar();
    switch(c){
      case 'w': walk();     break;
      case 'c': lcdcls(); break;
      case 't': term();     break;
      case 'v': vuloop();   break;
      case 'f': faders();   break;
    }
  }
}

//--------------------
void updatetod(void){
//update time of day
unsigned char tmptocs;

  tmptocs=tocs;     //from int handler
  deltat=tmptocs-tocsl; //1 ms tocs
  tocsl=tmptocs;
  
  t+=deltat; 
  osaccum+=deltat;
  if(osaccum > 1){
    os1sec=1;
	  osaccum-=1;
  }        
  if(os1sec){  //rt clock 
	  os1sec=0;
    secs++;
	  if(secs > 59){
	    secs=0;
	    mins++;
	    if(mins > 59){
	      mins=0;
		    hrs++;
		    if(hrs > 23){
		      hrs=0;
		    }//hr
	    }//min
	  }//sec
  }//os1sec
}

//----------------------
void timeloop(void){
//time of day
char c;

  while(c !='q'){  //loop till 'q'
    if(kbhit()){    //if char waiting from pc
	    c=getchar();
    }
    updatetod();
	  cprintf("%02d:%02d:%02d\r",hr,min,sec);
	  delnms(100);
  } //while
}

//---------------------------------------
#pragma interrupt_handler timer0_comp_isr:iv_TIMER0_COMP
void timer0_comp_isr(void){
//compare occured TCNT0=OCR0   1ms

  TCNT0=0x20;
  tocs++;
}

//-----------------------
void rtloop(void){
//rtloop using rs232
char c;

  while(c !='q'){  //loop till 'q'
    if(kbhit()){    //if char waiting from pc
	    c=getchar();
	    if(c=='t') teston = !teston; //also need to read bit button..
	    if(c=='p') {
	      dispon=!dispon;
		    if(dispon){
		    }
	    }
    }
	  passcnt++;
	  if(kphit()){
	    relayon();
	  }else{
	    relayoff();
	  }  
	
	  if(dispon){
	  }
  } //while
}

//--------------------------
void rs232menu(void){
//cmds

//  initscreen();
//  gotoxy(1,7);
  cprintf("rs232 cmds:\n");
  cprintf(" d debug cmds\n");
  cprintf(" a a/d\n");
  cprintf(" b buttons\n");
  cprintf(" l lcd cmds\n");
  cprintf(" t time\n");
  cprintf(" r rt loop\n");
}

//--------------------------
void mainswitch(char c){
//parse char from keyboard

  switch(c){
    case 'a': readadloop(); break;
    case 'b': butloop();    break;
    case 'l': lcdloop();    break;
    case 't': timeloop();   break;
    case 'd': debugloop();  break;
    case 'r': rtloop();     break;
    default: rs232menu();
  }
}

//-----------------
void main(void){
//main program
char c;

  init_devices();
  initscreen(); //rs232
  initvars();
  lcdinit();
  lcdcls();
  lcdcputs("MT 128 test");
  lcdgotoxy(1,2);
  lcdcputs(" 1/15/08");
  while(1){      //main loop
    rs232menu();
    crlf();
    putc('>'); //prompt for input
    c=getche();
    crlf();
    mainswitch(c);
    _StackCheck();
  }
}

//---strings embedded in rom--------------
const char copyrightstring[]={"avrmt128tst computer code by\
BobGardner at aol.com"};
//--------eof-------------

